% 
%  SRS Synthesis using Half-Sine Pulse with Wavelet Reconstruction
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
disp(' ');
disp(' halfsine_synth.m  ver 1.0  April 19, 2014 ');
disp(' by Tom Irvine');
disp(' Email:  tom@vibrationdata.com ');
disp(' ');
disp(' This program synthesizes a half-sine pulse to satisfy a shock');
disp(' response spectrum specification, with optional wavelet ');
disp(' reconstruction to yield zero net displacement and zero net velocity');
disp(' ');
%
close all;
%
clear a;
clear f;
clear ra;
clear rf;
clear rff;
clear slope;
clear THF;
clear tt;
clear store;
clear max;
clear THM;
clear xmin;
clear xmax;
clear resp;
%
clear acceleration;
clear velocity;
clear displacement;
clear accel_ds;
clear accel;
clear srs_syn;
%
fig_num=1;
%
iunit=1;
%
tic
%
MAX=400;
FMAX=400;
TMAX=100000;
%
tpi=2.*pi;
octave=(2.^(1./12.));
%
errlit=1.0e+90;
syn_error = 1.0e+91;  
%
f=zeros(MAX,1);
a=zeros(MAX,1);
slope=zeros(MAX,1);
%
while(1)
    disp(' Select units ');
    disp('  1=English: accel(G), vel(in/sec),  disp(in)  ');
    disp('  2=metric : accel(G), vel(m/sec),  disp(mm)  '); 
    iunit=input(' ');
    if(iunit==1 || iunit==2)
        break;
    end
end	
%
disp(' Select data input method.');
disp('  1=keyboard  ');
disp('  2=internal Matlab array  ');
disp('  3=external ASCII file   ');
imethod=input(' ');
disp(' ');
%  
if(imethod==1)
%
    n=0;
    while(n < 2)
        disp('  How Many Breakpoints (min=2)? ');
        n=input(' ');
    end
%  
    disp('  Enter Points (free-format)');
    disp('  Natural Freq(Hz)  SRS(G)   ');
%   
    for i=1:n
        out1=sprintf(' Enter natural frequency %d:  ',i);
        f(i)=input(out1);
        out2=sprintf(' Enter SRS(G) %d:  ',i);
        a(i)=input(out2);  
        disp(' ')
    end
%
end
%
if(imethod==2)
    disp('The array must have two columns:  Natural Freq(Hz)  SRS(G)   ');
    THM=input(' Enter the array name:  ');
    f=THM(:,1);
    a=THM(:,2);
    clear length;
    n=length(a);
end
%
if(imethod==3)
%
    [filename, pathname] = uigetfile('*.*');
    filename = fullfile(pathname, filename); 
    fid = fopen(filename,'r');
    THF = fscanf(fid,'%g %g',[2 inf]);
    THF=THF';   
    f=THF(:,1);
    a=THF(:,2);
    clear sz;
    sz=(size(f));
    n=sz(1);
    if(sz(2)>sz(1))
        n=sz(2);
    end
%
end
%
srs_spec=[f a];
%
ffirst=f(1);
flast=f(n);
last_f=f(n);
last_a=a(n);
%
ffmin=ffirst;
ffmax=flast;
%
%
for i=1:(n-1)
    slope(i)=log(a(i+1)/a(i))/log(f(i+1)/f(i));
end
%
%
dre=3/f(1);
disp(' ');
disp(' Enter total duration (sec) including quiet periods before & after pulse: ');
out1=sprintf(' (recommend >= %8.4g) ',dre);
disp(out1);
dur=input('  ');
%
%% first=0.15*dur;
first=0;
%
sr=10*last_f;
%
out1=sprintf(' \n Recommend sample rate = %12.6g samples/sec\n',sr);
disp(out1);
%
iasa=input(' Accept recommended rate?  1=yes 2=no ');
%
if(iasa~=1)
    disp(' ');
    sr=input(' Enter sample rate  ');
end
%
dt=1./sr;
ns=round(sr*dur);
%
if(ns > TMAX)
    disp('  Warning: Excessive number of samples. ');
end
%
out1=sprintf(' \n sample rate = %12.4g samples/sec\n ',sr);
disp(out1);
%
idamp=input(' Enter damping format:  1=damping ratio   2=Q ');
%
if(idamp == 1 )
    damp=input(' Enter damping ratio (typically 0.05) ');
else
    damp=input(' Enter amplification factor Q (typically 10) ');
    damp = 1./(2.*damp);
end
Q=1./(2.*damp);
%
iamax=input(' Enter number of iterations for outer loop (suggest 1000)');
%
if(iamax < 1 )
    iamax=1;
end
%
ntt=2;  % number of iterations for inner loop
%
out1=sprintf('\n  ns=%ld \n',ns);
disp(out1);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
disp(' ');
disp('  Interpolating reference. ');
%
rf=zeros(FMAX,1);
ra=zeros(FMAX,1);
%
rf(1)=f(1);
ra(1)=a(1);
%
for i=2:FMAX
%
    rf(i)=rf(i-1)*octave;
%
    if( rf(i) == max(f) )
    break;
    end
    if( rf(i) > max(f) )
        rf(i)=max(f);
        break;
    end
end
last=i;
nfr=last;
%
out1=sprintf(' \n last = %ld   flast = %12.4g\n',last,rf(last));
disp(out1);
%
for i=1:last
%
    for j=1:n
%
        if( rf(i) > f(j) && rf(i) <f(j+1))
            ra(i)=a(j)*( (rf(i)/f(j))^slope(j) );
            break;
        end
%
        if( rf(i)==f(j))
            ra(i)=a(j);
            break;
        end
%
        if( rf(i)==f(j+1))   
            ra(i)=a(j+1);
            break;
        end
    end
end
%
rf(last)=last_f;
ra(last)=last_a;
%
out1=sprintf(' \n last=%ld ',last);
disp(out1);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
jkj=1;
%
best_amp=zeros(last,1);
best_phase=zeros(last,1);
best_delay=zeros(last,1);
best_dampt=zeros(last,1);
%
freq=rf;
clear omega;
omega=tpi*rf;
%
maxA=max(srs_spec(:,2));
%
progressbar;
%
dd=0.6*dur;
%
for ia=1:iamax
    progressbar(ia/iamax);
    iflag=0;
%
    if(rand()>0.5 && ia>10)
        amp=best_amp*(0.98+0.04*rand());       
          T=best_T*(0.98+0.04*rand());
    else    
        amp=maxA/2;
          T=dd*rand();
    end
%
    if(amp<1)
        amp=1;
    end
    if(T>dd)
        T=dd;
    end
%   
%  put in the middle
%
    mh=floor(0.5*ns*T/dur);
    nh=floor(ns/2);
%
    n1=(nh-mh);
    n2=(nh+mh);
%    
    time_history=zeros(ns,1);
    for i=n1:n2
        t=(i-n1)*dt;
        g=amp*sin(t*pi/T);
        if(g>=0)
            time_history(i)=g;
        end    
    end    
%
%%    out1=sprintf(' amp=%8.4g T=%8.4g dur=%8.4g M=%d M+N=%d ns=%d  ',amp,T,dur,M,M+N,ns);
%%    disp(out1);
%
    [~,xmax]=HS_srs(FMAX,last,ns,freq,damp,dt,time_history);
%
    for ijk=1:ntt
%
        dB=zeros(last,1);
%        
        iflag=0; 
        for i=1:last
            xx = xmax(i);  % one-sided half-sine 
            if(xx <1.0e-90)
                iflag=1;
            end
            dB(i)=20*log10(xx/ra(i));  % reference to spec
 %       
        end
 %
        if(iflag==0)
            mindB=min(dB);
            maxdB=max(dB);
 %
            maxscale=abs(10^(maxdB/20));
            minscale=abs(10^(mindB/20));
 %
            if(mindB <0.)  % synth is under spec
                scale=1/minscale;
            else
                scale=minscale;
            end
%
            amp=amp*scale;
            time_history=time_history*scale;
%
%%            out1=sprintf('*   amp=%8.4g scale=%8.4g   mindB=%8.4g',amp,scale,mindB);
%%            disp(out1);
%
            if(iflag==1)
                disp(' ref 1 ');
                break;
            end
%
            if(max(time_history)>0)
%
                [xmin,xmax]=HS_srs(FMAX,last,ns,freq,damp,dt,time_history);
%
                [syn_error,iflag]=HS_srs_error(last,xmax,xmin,ra,iflag);
%
                if(iflag==1)
                    disp(' ref 2 ');
                    break;
                end
%
                if( (syn_error < errlit) || ia==1)
%
                    iacase =ia;
                    icase = ijk;
%
                    errlit = syn_error;
%
                    out1=sprintf(' %ld %ld  error= %12.4g  amp=%8.4g  T=%8.4g',ia,...
                                                        ijk,syn_error,amp,T);
                    disp(out1)
%           
                    best_amp=amp;
                    best_T=T;
%
                    store=time_history;
%
                end
%
                if(ijk>8 && syn_error > error_before)  
                    break;
                end
                if(ijk>8)  
                    break;
                end   
                if(ijk>1 && syn_error > 1.0e+90)
                    break;
                end
%
                error_before=syn_error;
            end
%%%            disp(' ');
%
            if(jkj==1)
                jkj=2;
            else
                jkj=1;
            end
        end   
     end
end
pause(0.5);
progressbar(1); 
%
out1=sprintf(' \n\n Best case is %ld %ld ',iacase,icase);
disp(out1);
%
store;
%
[xmin,xmax]=HS_srs(FMAX,last,ns,freq,damp,dt,store);
%
freq=fix_size(freq);
xmin=fix_size(xmin);
xmax=fix_size(xmax);
%
clear length;
nnn=length(xmax);
ff=freq(1:nnn);
clear freq;
freq=ff;
%
srs_syn=[freq xmin xmax];
%
clear tr;
clear store_recon;
%
store_recon=store;
%
%
[v]=integrate_function(store,dt);
%
[d]=integrate_function(v,dt);
%
tt=linspace(0,(ns-1)*dt,ns);
tt=fix_size(tt);
tr=tt;
store=fix_size(store);
v=fix_size(v);
d=fix_size(d);
%
acceleration=[tt store];
velocity=[tt v];
displacement=[tt d];
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
disp(' ');
disp(' Perform waveform reconstruction for zero net velocity & zero net displacement? ');
disp(' 1=yes  2=no ');
wc=input(' ');
%
if(wc==1)
%
    nt=ns;  
    [acceleration,velocity,displacement,srs_syn,wavelet_table]=...
    HS_waveform_reconstruction(tt,store,dt,first,freq,...
                                             ffmin,ffmax,damp,iunit,nt,nfr);
%    
end
toc
%
[fig_num]=...
plot_avd_srs_damped_sine(acceleration,velocity,displacement,srs_syn,...
                                              srs_spec,damp,fig_num,iunit);
%
%%%%%%%%%%%%%%%%%%%%
%
clear shock_response_spectrum;
%
shock_response_spectrum=srs_syn;
%
disp(' ');
disp(' Matlab array names: ');
disp('  ');
disp('   acceleration ');
disp('   velocity ');
disp('   displacement  ');
disp('   shock_response_spectrum  ');
disp('   wavelet_table ');
%
%%%%%%%%%%%%%%%%%%%%
%
disp(' ');
disp(' Export to ASCII text ');
disp(' 1=yes  2=no ');
et=input(' ');
if(et==1)
%
        out1=sprintf('\n Enter the acceleration time history output filename: ');
        disp(out1);
        output_filename1 = input(' ','s');
        out1=sprintf('\n Enter the velocity time history output filename: ');
        disp(out1);
        output_filename2 = input(' ','s');
%
        out1=sprintf('\n Enter the displacement time history output filename: ');
        disp(out1);
        output_filename3 = input(' ','s');
%
        out1=sprintf('\n Enter the shock response spectrum output filename: ');
        disp(out1);
        output_filename4 = input(' ','s');
%
        if(wc==1)
            out1=sprintf('\n Enter the wavelet table output filename: ');
            disp(out1);
            output_filename5 = input(' ','s');
        end
%
        save(output_filename1,'acceleration','-ASCII')
        save(output_filename2,'velocity','-ASCII')
        save(output_filename3,'displacement','-ASCII')
        save(output_filename4,'shock_response_spectrum','-ASCII')
        if(wc==1)
            save(output_filename5,'wavelet_table','-ASCII')
        end
%
end
%
disp(' ');
disp('  Best Case without Reconstruction ');
out1=sprintf('  Half-sine Pulse: %8.4g sec  %8.4g G',best_T,best_amp);
disp(out1);